unit network;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IdBaseComponent, IdComponent, IdTCPConnection,
  IdTCPClient, ExtCtrls, IdAntiFreezeBase, IdAntiFreeze, JvAppStorage,
  JvAppIniStorage, JvComponent, JvFormPlacement;

type
  TNetForm = class(TForm)
    ExecMemo: TListBox;
    Traffic: TListBox;
    ChatMem: TListBox;
    ServerEventTimer: TTimer;
    ServerMsg: TPanel;
    AntiFreeze: TIdAntiFreeze;
    Client: TIdTCPClient;
    JvFormStorage1: TJvFormStorage;
    NetStor: TJvAppIniFileStorage;
    procedure ServerEventTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ClientConnected(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TrafficDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ExecMemoDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ChatMemDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  NetForm: TNetForm;

Const
  VER_NET = '1.0 (build 100)';

Function WriteNet(what: String; coded, format: Boolean): String;
Procedure SwitchNetwork(online: Boolean);

implementation
Uses LoadForm, Logger;

{$R *.dfm}

Procedure SwitchNetwork(online: Boolean);
const
  PN = 'SwitchNetwork';

Begin
  With NetForm do
  Begin
    If online then
    Begin
      WriteLog(PN, LT_debug, ET_NoErr, 'Attempting network establishment.');
      Client.Connect;
      netform.Show;
    End Else
    Begin
      If not Client.Connected then Exit;
      WriteLog(PN, LT_Debug, ET_NoErr, 'Shutting down network connection.');
      Client.WriteLn('?001');
      Client.Disconnect;
      netform.Close;
    End;
    online:= Client.Connected;
    ServerEventTimer.Enabled:= online;
  End;
End;


Function WriteNet(what: String; coded, format: Boolean): String;
var msg: String;
    x:   Byte;
    h:   String;
Begin
  With NetForm do
  Begin
    If not Client.Connected then
    Begin
      Result:= 'Not connected';
      Exit;
    End;
    Client.WriteLn(what);
    msg:= Client.ReadLn;
    {Wenn es formatiert werden soll, dann ist hier die Formatierung}
    If format then
    Begin
      If (AnsiPos('?005', what) <> 0) and (AnsiPos('!005', msg) <> 0)
      then msg:= 'Classified';
      If AnsiPos('?007', what) <> 0 then Delete(msg,1,4);
    End;
    {Resultat bergeben und evt. den Ausgang codiert angeben}
    Result:= msg;
    If coded then
      For x:= 5 to Length(what)
        do What[x]:= '*';
    If what <> '?007' then
    Begin
      Traffic.Items.Add('> '+what);
      Traffic.Items.Add('< '+msg);
    End;
    {Hier schreiben wir das Ganze noch in die ExecMemo zur besseren VErstndnis}
    h:= what;
    Delete(h,5,length(h));
    Delete(h,1,1);
    If ExecMemo.Items.Count > 500 then ExecMemo.Items.Clear;
    If Traffic.Items.Count > 1000 then Traffic.Items.Clear;
    If what <> '?007' then
    Case StrToInt(h) of
      001: Begin
             ExecMemo.Items.Add('1Requesting Server-sided disconnection...');
             If msg = 't'
             then ExecMemo.Items.Add('tServer has agreed and disconnected.')
             Else ExecMemo.Items.Add('fServer has refused double-sided disconnection, forcing it here.');
           End;
      002: Begin
             ExecMemo.Items.Add('1Requesting connection to server...');
             If msg = 'true'
             then ExecMemo.Items.Add('tServer has accepted connection.')
             Else ExecMemo.Items.Add('fServer has refused connection.');
           End;
      003: ExecMemo.Items.Add('2Requesting server version number.');
      004: Begin
             ExecMemo.Items.Add('2Requesting my DNS name from server...');
             If msg[1] <> '!'
             then ExecMemo.Items.Add('tDNS sent: '+msg)
             Else ExecMemo.Items.Add('fSending has been refused by server.');
           End;
      005: Begin
             ExecMemo.Items.Add('3Requesting my server-sided status code from server...');
             If msg[1] <> '!'
             then ExecMemo.Items.Add('tCode sent: '+msg)
             Else ExecMemo.Items.Add('fSending has been refused by server.');
           End;
      006: Begin
             ExecMemo.Items.Add('3Requesting my session ID from server...');
             If msg[1] <> '!'
             then ExecMemo.Items.Add('tSession ID sent: '+msg)
             Else ExecMemo.Items.Add('fSending has been refused by server.');
           End;
      008: Begin
             ExecMemo.Items.Add('2Answering verification request...');
             If msg[1] = 't'
             then ExecMemo.Items.Add('tVerification successfull.')
             Else ExecMemo.Items.Add('fVerification failed. You may be disconnected by server.');
           End;
      009: Begin
             ExecMemo.Items.Add('2Requesting full server information...');
             If (msg[1] = '!') and (msg[3] = '1')
             then ExecMemo.Items.Add('tInformation received.')
             Else ExecMemo.Items.Add('fInformation denied by server.');
           End;
      010: Begin
             ExecMemo.Items.Add('3Logging in as Admin and requesting full rights...');
             If msg[1] = 't'
             then ExecMemo.Items.Add('tLogin Accepted.')
             Else ExecMemo.Items.Add('fLogin denied. You might be disconnected.');
           End;
      011: Begin
             ExecMemo.Items.Add('3Logging in as Admin, phase 2...');
             If msg[1] = 't'
             then ExecMemo.Items.Add('tOK, you are an Admin.')
             Else ExecMemo.Items.Add('fWrong credentials given. You might be disconnected.');
           End;
      051: Begin
             ExecMemo.Items.Add('3Requesting name change.');
             If msg[1] = 't'
             then ExecMemo.Items.Add('tChange successfull.');
           End;
      052: ExecMemo.Items.Add('2Requesting my name from server.');
      100: ExecMemo.Items.Add('2Sending Log Line to server.');
      101: ExecMemo.Items.Add('!Sending full information package.');
      102: Begin
             ExecMemo.Items.Add('!Sending chat to Server.');
             h:= What;
             Delete(h,1,4);
             ChatMem.Items.Add('You ['+TimeToStr(Time)+']: '+h);
           End;
      200: ExecMemo.Items.Add('2Requesting Server time.');
      201: ExecMemo.Items.Add('2Requesting Server date.');
      Else ;
    End;
    If Length(msg) >= 4 then
      if ((msg[1] = '!') and (msg[4] in ['0'..'9'])) and (what <> '?007') then
      Begin
        h:= msg;
        While h[1] = '!' do Delete(h,1,1);
        Delete(h,4,length(h));
        Case StrToInt(h) of
          000: ExecMemo.Items.Add('4Server does not understand the command.');
          001: ExecMemo.Items.Add('4Server does not understand the format of the command.');
          002: ExecMemo.Items.Add('4Wrong identification. You will be disconnected.');
          009: ExecMemo.Items.Add('5Option disabled by Server.');
          101: ExecMemo.Items.Add('5Server requesting full information package.');
          102: ExecMemo.Items.Add('5Server sending a chat line.');
        End;
      End;
  End;
End;

Procedure EvaluateStatusResponse(what: String);
var cmd:   Integer;
    h:     String;
    x:     Word;
    break: Integer;
Begin
  If what[1] <> '!' then
  Begin
    Exit;
  End;
  Delete(what,1,1);
  break:= 100000000;
  While AnsiPos('!', what) <> 0 do
  Begin
    Delete(what, 1,1);
    cmd:= StrToInt(what[1] + what[2] +what[3]);
    Delete(what, 1, 3);
    x:= AnsiPos('!', what);
    h:= what;
    netform.Traffic.Items.Add(h);
    If x > 0 then Delete(h,x,Length(h));
    With netform do
    Case cmd of
      006: If (StrToInt(h) < break) or (StrToInt(h) < 5) then
           If AnsiPos('002',h) <> 0 then
           Begin
             Delete(h,1,3);
             If StrToInt(h) < 5 then
             Begin
               SwitchNetwork(false);
               Traffic.Items.Add('');
               Traffic.Items.Add('Server connection was shut down '+h+'s before server-sided connection cut.');
               Exit;
             End;
             ServerMsg.Visible:= True;
             ServerMsg.Caption:= 'Kicked from the server in '+h+'s ...';
             break:= StrToInt(h);
           End;
      001: WriteNet('?008'+h, False, False);
      004: If (StrToInt(h) < break) or (StrToInt(h) < 3) then
           Begin
             If StrToInt(h) < 3 then
             Begin
               SwitchNetwork(false);
               Traffic.Items.Add('');
               Traffic.Items.Add('You were kicked from the server because the server was shutting down.');
               Exit;
             End;
             ServerMsg.Visible:= True;
             ServerMsg.Caption:= 'Server Shutdown in '+h+'s ...';
             break:= StrToInt(h);
           End;
      100: WriteNet('?101?'+VER_GLON+'?Star Trek Ship Constructor?Zortiander?'+VER_NET+
                    '?STSC Client?Zortiander', False, False);
      102: Begin
{             CloseLbl.Visible:= True;
             ImpMsgPanel.Visible:= True;
             ImpMsgPanel.Caption:= what;}
             ChatMem.Items.Add('Server ['+TimeToStr(Time)+']: '+what);
           End;
    End;
    If x > 0 then Delete(what,1,x-1);
  End;
End;

procedure TNetForm.ServerEventTimerTimer(Sender: TObject);
begin
  If not (Client.Connected) then
  Begin
    SwitchNetwork(false);
    ServerEventTimer.Enabled:= False;
    Exit;
  End;
  EvaluateStatusResponse(WriteNet('?007', False, False));
end;

procedure TNetForm.FormCreate(Sender: TObject);
begin
  If not FileExists('dev.txt') then Exit;
  SwitchNetwork(true);
end;

procedure TNetForm.ClientConnected(Sender: TObject);
var h: String;
begin
  WriteLog('Client.Connected', LT_debug, ET_NoErr, 'Requesting Server connection.');
  h:= netform.Client.ReadLn;
  h:= WriteNet('?002dkpN23v6z', True, False);
end;

procedure TNetForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SwitchNetwork(false);
end;

procedure TNetForm.TrafficDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var h: String;
begin
  If Index < 0 then Exit;
  h:= Traffic.Items[Index];
  if Length(h) < 2 then Exit;
  With Traffic.Canvas do
  Begin
    Font.Color:= clBlack;
    If h[1] = '>' then Font.Color:= clBlue;
    If h[1] = '<' then Font.Color:= clMaroon;
    If h[1] in ['>','<'] then Delete(h,1,1);
    FillRect(Rect);
    TextOut(Rect.Left, Rect.Top, h);
  End;
end;

procedure TNetForm.ExecMemoDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  h:    String;

begin
  h:= ExecMemo.Items[Index];
  If Length(h) < 2 then Exit;
  With ExecMemo.Canvas.Font do
    Case h[1] of
      '!': Color:= clBlack;
      '1': Color:= clLime;
      '3': Color:= clPurple;
      't': Color:= clGreen;
      'f': Color:= clMaroon;
      '5': Color:= clNavy;
      '?': Color:= clBlue;
      Else Color:= clGray;
    End;
  Delete(h,1,1);
  ExecMemo.Canvas.FillRect(Rect);
  ExecMemo.Canvas.TextOut(Rect.Left, Rect.Top, h);
end;

procedure TNetForm.ChatMemDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
begin
  With ChatMem.Canvas do
  Begin
    Case ChatMem.Items[Index][1] of
      'S': Font.Color:= clBlack;
      'Y': Font.Color:= clGray;
    End;
    FillRect(Rect);
    TextOut(Rect.Left, Rect.Top, ChatMem.Items[Index]);
  End;
end;

end.
